package ch.zhaw.gartenverwaltung.plantList;

import ch.zhaw.gartenverwaltung.types.Plant;

import java.util.Comparator;
import java.util.List;
import java.util.function.Function;
import java.util.function.Predicate;

public class PlantListModel {
    private List<Plant> plantList;

    /**
     * Comparators to create sorted Plant List
     */
    public final Comparator<Plant> sortByName = (Plant o1, Plant o2) -> o1.name().compareTo(o2.name());
    public final Comparator<Plant> getSortById = (Plant o1, Plant o2) -> Long.compare(o1.id(), o2.id());
    public final Comparator<Plant> sortBySpacing = (Plant o1, Plant o2) -> o1.spacing() - o2.spacing();

    /**
     * Functions to get Plant Attribute as String
     */
    public final Function<Plant, String> filterByName = Plant::name;
    public final Function<Plant, String> filterById = plant -> Long.toString(plant.id());


    public PlantListModel(List<Plant> plantList) {
        setPlantList(plantList);
    }

    public void setPlantList(List<Plant> plantList) {
        this.plantList = plantList;
    }

    /**
     * Method to get actual Plant List in alphabetic Order
     * @return actual Plant List in alphabetic Order
     */
    public List<Plant> getPlantList() {
        return getSortedPlantList(sortByName);
    }

    /**
     * Method to get actual Plant List sorted in custom order
     * @param comparator comparator Object which is used to sort list
     * @return actual Plant List in custom order
     */
    public List<Plant> getSortedPlantList(Comparator<Plant> comparator) {
        return plantList.stream().sorted(comparator).toList();
    }

    /**
     * Method to get filtered Plant List with custom filter
     * @param filterString String to search for plants
     * @param filterFunction Function Object to get Plant attribute as String which must contain filter String
     * @return filtered Plant List
     */
    public List<Plant> getFilteredPlantListbyString(String filterString, Function<Plant, String> filterFunction) {
        return plantList.stream().filter(plant -> filterFunction.apply(plant).contains(filterString)).toList();
    }
}
